import React from "react";
import { canUseDOM } from "exenv-es6";
import { defaultBreakpoints, identifyBreakpoint } from "./breakpoints";
export default class BreakpointTracker extends React.Component {
    /**
     * Constructor for the BreakpointTracker component.
     */
    constructor(props) {
        super(props);
        /**
         * Updates the active breakpoint
         */
        this.updateBreakpoint = () => {
            const windowWidth = window.innerWidth;
            const breakpoint = identifyBreakpoint(windowWidth, BreakpointTracker.breakpoints);
            if (this.state.activeBreakpoint !== breakpoint) {
                this.setState({
                    activeBreakpoint: breakpoint,
                });
            }
            this.openRequestAnimationFrame = false;
        };
        /**
         * Request's an animation frame if there are currently no open animation frame requests
         */
        this.requestFrame = () => {
            if (!this.openRequestAnimationFrame) {
                this.openRequestAnimationFrame = true;
                window.requestAnimationFrame(this.updateBreakpoint);
            }
        };
        this.state = {
            activeBreakpoint: null,
        };
    }
    /**
     * React life-cycle method
     */
    componentDidMount() {
        // We are doing this work in a lifecycle method instead of the constructor to ensure that
        // server rendered instances with conditional DOM rendering by breakpoint will be re-rendered.
        // ReactDOM.hydrate() will call the constructor again, but it does not trigger a re-render. It will only bind event handlers.
        // The only way to ensure the correct DOM is consistently rendered on the client is to perform this work here.
        if (canUseDOM()) {
            this.updateBreakpoint();
            window.addEventListener("resize", this.requestFrame);
        }
    }
    /**
     * React life-cycle method
     */
    componentWillUnmount() {
        if (canUseDOM()) {
            window.removeEventListener("resize", this.requestFrame);
        }
    }
    /**
     * React render method
     */
    render() {
        return this.props.render(this.state.activeBreakpoint);
    }
}
/**
 * The array of breakpoint values
 */
BreakpointTracker.breakpoints = defaultBreakpoints;
